﻿using System;
using System.Windows.Data;
using System.Windows;

namespace Monahrq.Theme.Converters
{
	/// <summary>
	/// Converts null to Visibility.
	/// </summary>
	/// <seealso cref="System.Windows.Data.IValueConverter" />
	public class NullToVisibilityConverter : IValueConverter
  {
		#region IValueConverter Members

		/// <summary>
		/// Converts a value.
		/// </summary>
		/// <param name="value">The value produced by the binding source.</param>
		/// <param name="targetType">The type of the binding target property.</param>
		/// <param name="parameter">The converter parameter to use.</param>
		/// <param name="culture">The culture to use in the converter.</param>
		/// <returns>
		/// A converted value. If the method returns null, the valid null value is used.
		/// </returns>
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
    {
      if (parameter == null) //Not invert parameter passed
      {
        return (value == null) ? Visibility.Collapsed : Visibility.Visible;
      }
      else if (parameter.ToString() == "Invert")
      {
        return (value == null) ? Visibility.Visible : Visibility.Collapsed;
      }
      return Visibility.Collapsed;
    }

		/// <summary>
		/// Converts a value.
		/// </summary>
		/// <param name="value">The value that is produced by the binding target.</param>
		/// <param name="targetType">The type to convert to.</param>
		/// <param name="parameter">The converter parameter to use.</param>
		/// <param name="culture">The culture to use in the converter.</param>
		/// <returns>
		/// A converted value. If the method returns null, the valid null value is used.
		/// </returns>
		/// <exception cref="NotImplementedException"></exception>
		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
    {
      throw new NotImplementedException();
    }

    #endregion
  }
}
