﻿using System;
using System.Globalization;
using System.Windows;
using System.Windows.Data;

namespace Monahrq.Measures.Converters
{
	/// <summary>
	/// Returns visibility bases on input count being greater than 1.
	/// </summary>
	/// <seealso cref="System.Windows.Data.IValueConverter" />
	public class ZeroFilterVisibilityConverter : IValueConverter 
    {
		/// <summary>
		/// Converts a value.
		/// </summary>
		/// <param name="value">The value produced by the binding source.</param>
		/// <param name="targetType">The type of the binding target property.</param>
		/// <param name="parameter">The converter parameter to use.</param>
		/// <param name="culture">The culture to use in the converter.</param>
		/// <returns>
		/// A converted value. If the method returns null, the valid null value is used.
		/// </returns>
		public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            var count = Int32.Parse(value.ToString());
            return count < 1 ? Visibility.Visible : Visibility.Collapsed;
        }

		/// <summary>
		/// Converts a value.
		/// </summary>
		/// <param name="value">The value that is produced by the binding target.</param>
		/// <param name="targetType">The type to convert to.</param>
		/// <param name="parameter">The converter parameter to use.</param>
		/// <param name="culture">The culture to use in the converter.</param>
		/// <returns>
		/// A converted value. If the method returns null, the valid null value is used.
		/// </returns>
		/// <exception cref="NotImplementedException"></exception>
		public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
