/* =============================================
-- Author:		Shafiul Alam
-- Project:		MONAHRQ 6.0 Build 1
-- Create date: 12-22-2014
-- Description:	This is the update script from older MONAHRQ 5.0 Reports table to the new 
--              MONAHRQ 6.0 database schema.
--				'Reports Data'
-- =============================================*/

BEGIN TRY 

IF OBJECT_ID('tempdb..#ReportsForeignKeys') IS NOT NULL
DROP TABLE #ReportsForeignKeys

SELECT identity(INT,1,1) rownum,
OBJECT_NAME(fk.parent_object_id) parentTable,NAME 
INTO #ReportsForeignKeys
FROM sys.foreign_keys fk
WHERE fk.referenced_object_id= OBJECT_ID('Reports')

DECLARE @LoopIterator INT=1,
		@LoopCount INT= (SELECT COUNT(1) FROM #ReportsForeignKeys),
		@sql VARCHAR(8000)
		
WHILE (@LoopIterator<=@LoopCount)
BEGIN
	
	SELECT @sql='ALTER TABLE [dbo].['+parentTable+'] DROP CONSTRAINT '+NAME
	FROM #ReportsForeignKeys i
	WHERE rownum=@LoopIterator
	
	EXEC (@sql)
	
	SELECT @sql='IF NOT EXISTS (SELECT 1 FROM sys.[columns] c WHERE c.name=''ReportsId'' AND c.[object_id]=OBJECT_ID('''+parentTable+'''))
	             ALTER TABLE [dbo].['+parentTable+'] ADD ReportsId INT'
	FROM #ReportsForeignKeys i
	WHERE rownum=@LoopIterator
	
	EXEC (@sql)
	
	SET @LoopIterator=@LoopIterator+1
END

IF NOT EXISTS (SELECT 1 FROM sys.[columns] c WHERE c.name='ReportsId' AND c.[object_id]=OBJECT_ID('Reports'))
ALTER TABLE Reports add ReportsId INT IDENTITY(1,1)

/*******************************************
 *  Updating new Identity Report Id to
 *  Foreign key columns
 *******************************************/

SET @LoopIterator=1

WHILE (@LoopIterator<=@LoopCount)
BEGIN
	
	SELECT @sql='UPDATE  [dbo].['+parentTable+'] SET ReportsId = r.ReportsId
                 FROM [dbo].['+parentTable+'] a
				 INNER JOIN Reports r
				 ON r.Id = a.Report_Id'
	FROM #ReportsForeignKeys i
	WHERE rownum=@LoopIterator
	
	EXEC (@sql)
	
	SET @LoopIterator=@LoopIterator+1
END

/*******************************************
 *  Drop,Rename and create Primary Key in Reports table
 *******************************************/

IF EXISTS (SELECT 1 FROM sys.[columns] c WHERE c.name='ID' AND c.[object_id]=OBJECT_ID('Reports'))
BEGIN
	IF EXISTS (SELECT 1 FROM sys.key_constraints kc WHERE object_name(kc.parent_object_id)='Reports' AND kc.[type]='PK')
	BEGIN	
		SELECT @sql='ALTER TABLE Reports DROP CONSTRAINT '+kc.name
		  FROM sys.key_constraints kc WHERE object_name(kc.parent_object_id)='Reports' AND kc.[type]='PK'
		EXEC (@sql)  
	END
ALTER TABLE Reports DROP COLUMN Id;
EXEC sp_rename 'dbo.Reports.ReportsId', 'Id', 'COLUMN';
ALTER TABLE Reports ADD PRIMARY KEY (Id)

IF(OBJECT_ID(N'ReportsTemp')) IS NOT NULL	
	DROP TABLE ReportsTemp

CREATE TABLE ReportsTemp(
	[Id] [INT] IDENTITY(1,1) NOT NULL,
	[Name] [NVARCHAR](255) NOT NULL,
	[ReportProfile] [NVARCHAR](255) NULL,
	[Category] [NVARCHAR](255) NULL,
	[IsTrending] [BIT] NULL,
	[Audiences] [NVARCHAR](255) NOT NULL,
	[Datasets] [NVARCHAR](MAX) NOT NULL,
	[RequiresCostToChargeRatio] [BIT] NULL,
	[RequiresCmsProviderId] [BIT] NULL,
	[ReportAttributes] [NVARCHAR](255) NULL,
	[Description] [NVARCHAR](MAX) NULL,
	[Footnote] [NVARCHAR](MAX) NULL,
	[ComparisonKeyIconSetName] [NVARCHAR](255) NULL,
	[ReportType] [NVARCHAR](255) NOT NULL,
	[SourceTemplateXml] [XML] NULL,
	[IsDefaultReport] [BIT] NOT NULL DEFAULT(0),
	[IsCustom] [BIT] NOT NULL  DEFAULT(0),
	[Filter] [NVARCHAR](255) NULL,
	[FilterItemsXml] [XML] NULL,
	[InterpretationText] [NVARCHAR](MAX) NULL,
	[ShowInterpretationText] [BIT] NOT NULL  DEFAULT(0),
	[ReportSql] [NVARCHAR](MAX) NULL,
PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]

SET IDENTITY_INSERT dbo.ReportsTemp ON 

INSERT INTO	dbo.ReportsTemp
        (Id ,Name ,ReportProfile ,Category ,IsTrending ,Audiences ,Datasets ,RequiresCostToChargeRatio ,
          RequiresCmsProviderId ,ReportAttributes ,Description ,Footnote ,ComparisonKeyIconSetName ,
          ReportType ,SourceTemplateXml ,IsDefaultReport ,IsCustom ,Filter ,FilterItemsXml ,
          InterpretationText ,ShowInterpretationText  
        )
SELECT	 Id ,Name ,ReportProfile ,Category ,0 ,Audiences ,Datasets ,RequiresCostToChargeRatio ,
          RequiresCmsProviderId ,ReportAttributes ,Description ,Footnote ,ComparisonKeyIconSetName ,
          SourceTemplateXml.value(N'/ReportManifest[1]/@Name',  'nvarchar(max)') ,SourceTemplateXml ,IsDefaultReport ,0 ,Filter ,FilterItemsXml ,
          InterpretationText ,ShowInterpretationText  
FROM [dbo].[Reports]

SET IDENTITY_INSERT dbo.ReportsTemp OFF	

IF(OBJECT_ID(N'Reports')) IS NOT NULL
	DROP TABLE [dbo].[Reports]

EXEC sp_rename  [ReportsTemp], [Reports]

END

/*******************************************
 *  Add foreign key
 *******************************************/

SET @LoopIterator=1

WHILE (@LoopIterator<=@LoopCount)
BEGIN
	 
	SELECT @sql='IF (NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N''[dbo].['+NAME+']'') AND parent_object_id = OBJECT_ID(N''[dbo].['+parentTable+']''))
	             AND EXISTS (SELECT 1 FROM sys.[columns] c WHERE c.name=''ReportsId'' AND c.[object_id]=OBJECT_ID('''+parentTable+''')))
				BEGIN
				ALTER TABLE '+parentTable+' DROP COLUMN Report_Id;
				EXEC sp_rename ''dbo.'+parentTable+'.ReportsId'', ''Report_Id'', ''COLUMN'';	
				ALTER TABLE [dbo].['+parentTable+']  WITH CHECK ADD  CONSTRAINT ['+NAME+'] FOREIGN KEY([Report_Id])
				REFERENCES [dbo].[Reports] ([Id])
				ALTER TABLE [dbo].['+parentTable+'] CHECK CONSTRAINT ['+NAME+']
				END'
	FROM #ReportsForeignKeys i
	WHERE rownum=@LoopIterator
	
	EXEC (@sql)
	
	SET @LoopIterator=@LoopIterator+1
END

--UPDATE Reports
--SET    ReportType = NAME;

/*******************************************
 *  Alter Reports_ComparisonKeyIconSets column
 *  from VARBINARY to NVARCHAR
 *******************************************/

ALTER TABLE Reports_ComparisonKeyIconSets ALTER COLUMN [BestImage] NVARCHAR(255)
ALTER TABLE Reports_ComparisonKeyIconSets ALTER COLUMN [BelowImage] NVARCHAR(255)
ALTER TABLE Reports_ComparisonKeyIconSets ALTER COLUMN [BetterImage] NVARCHAR(255)
ALTER TABLE Reports_ComparisonKeyIconSets ALTER COLUMN [NotEnoughDataImage] NVARCHAR(255)
ALTER TABLE Reports_ComparisonKeyIconSets ALTER COLUMN [AverageImage] NVARCHAR(255)

/*******************************************
 * Reports_ReportColumns ID column datatype change
 * to INT from UNIQUEIDENTIFIER
 *******************************************/

IF EXISTS (SELECT 1 FROM sys.[columns] c WHERE c.name='Id' AND c.[object_id]=OBJECT_ID('Reports_ReportColumns') 
           AND c.system_type_id=36 )
BEGIN
	IF EXISTS (SELECT 1 FROM sys.key_constraints kc WHERE object_name(kc.parent_object_id)='Reports_ReportColumns' AND kc.[type]='PK')
	BEGIN	
		SELECT @sql='ALTER TABLE Reports_ReportColumns DROP CONSTRAINT '+kc.name
		  FROM sys.key_constraints kc WHERE object_name(kc.parent_object_id)='Reports_ReportColumns' AND kc.[type]='PK'
		EXEC (@sql)  
	END
ALTER TABLE Reports_ReportColumns DROP COLUMN Id
ALTER TABLE Reports_ReportColumns add Id INT IDENTITY(1,1)
ALTER TABLE Reports_ReportColumns ADD PRIMARY KEY (Id)
END


MERGE INTO [dbo].[Reports] r 
 USING(VALUES('Avoidable Stays Maps by County Profile Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Avoidable Stays Maps by County Profile Report" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F831" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="AvoidableStaysMaps-county-profile.png" RequiresCmsProviderId="true" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;How do I interpret this table?&lt;/P&gt;
    &lt;p&gt;This table presents data on a selected quality indicator by county.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Numerator:&lt;/b&gt; The number of hospital stays for county residents for whom the event or outcome of interest occurred (e.g., death caused by heart attack).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Denominator:&lt;/b&gt; The number of county residents based on US Census Bureau data.  &lt;br/&gt;&lt;br/&gt;&lt;b&gt;Observed rate:&lt;/b&gt; The numerator divided by the denominator. This rate is unadjusted.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Risk adjusted rate:&lt;/b&gt; Risk-adjustment is a statistical process that adjusts a hospital''s estimated performance if the hospital had an "average" case mix.  Risk adjustment is performed as in AHRQ''s Quality Indicator software for Window.  For indicators that are not risk adjusted, a dash ("-") appears.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;10%, 20%, 30%, 40%, 50%:&lt;/b&gt;&lt;/P&gt;&lt;/b&gt; The amount displayed reflects the cost savings result of the identified percentage reduction in the number of avoidable stays for all hospitals in a county for the conditions (multiple) selected.&lt;/p&gt;</InterpretationText><ReportAttributes>ReportColumns</ReportAttributes><Description>This is a description</Description><IconSets /><Filters /><Datasets><Dataset Name="AHRQ-QI Area Data" /></Datasets><Columns><Column Id="0" Name="Numerator" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Denominator" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Rates" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Cost Savings" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column></Columns></ReportManifest>'),('Avoidable Stays Maps by County Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Avoidable Stays Maps by County Report" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F832" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="AvoidableStaysMaps-county.png" RequiresCmsProviderId="true" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;How do I interpret this table?&lt;P&gt;
	 &lt;p&gt;This table presents data on a selected quality indicator by county.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Numerator:&lt;/b&gt; The number of hospital stays for county residents for whom the event or outcome of interest occurred (e.g., death caused by heart attack).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Denominator:&lt;/b&gt; The number of county residents based on US Census Bureau data.  &lt;br/&gt;&lt;br/&gt;&lt;b&gt;Observed rate:&lt;/b&gt; The numerator divided by the denominator. This rate is unadjusted.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Risk adjusted rate:&lt;/b&gt; Risk-adjustment is a statistical process that adjusts a hospital\&amp;#146;s estimated performance if the hospital had an &amp;quot;average&amp;quot; case mix.  Risk adjustment is performed as in AHRQ\&amp;#146;s Quality Indicator software for Window.  For indicators that are not risk adjusted, a dash (&amp;quot;-&amp;quot;) appears.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;10%, 20%, 30%, 40%, 50%:&lt;/b&gt; The amount displayed reflects the cost savings result of the identified percentage reduction in the number of avoidable stays for all hospitals in a county for the conditions (multiple) selected.&lt;br /&gt;&lt;br /&gt;&lt;/p&gt;</InterpretationText><ReportAttributes>ReportColumns</ReportAttributes><IconSets /><Filters /><Datasets><Dataset Name="AHRQ-QI Area Data" /></Datasets><Columns><Column Id="0" Name="Numerator" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Denominator" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Rates" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Cost Savings" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column></Columns></ReportManifest>'),('Avoidable Stays Maps','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Avoidable Stays Maps" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F833" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="AvoidableStaysMaps.png" RequiresCmsProviderId="true" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="Consumers" /><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;How do I interpret the maps?&lt;/p&gt;
	&lt;p&gt;The maps show the number of hospital stays in each county for each 1,000, 10,000, or 100,000 county residents.  Darker colors represent higher rates, and lighter colors represent lower rates.  Counties are labeled with numbers with the county names below the map.&lt;br/&gt;&lt;br/&gt;Map colors are assigned based on quintiles.  When there are not enough data to report a value, the map color is grey.&lt;br/&gt;&lt;br/&gt;&lt;a href="#/resources/Methods" &gt;Learn more about the county rates.&lt;/a&gt;&lt;/p&gt;</InterpretationText><ReportAttributes>None</ReportAttributes><IconSets /><Filters /><Datasets><Dataset Name="AHRQ-QI Area Data" /></Datasets><Columns /></ReportManifest>'),('County Rates Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="County Rates Report" RptId="5AAF7FBA-7102-4C66-8598-A70597E2F825" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="CountyRates.PNG" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;P&gt;What are county rates of hospital use?&lt;/P&gt;	
	&lt;p&gt;Information is provided for each selected county by condition or procedure grouping. Select the titles in the top row to sort the results. Tables may include the following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US&lt;/b&gt;: National numbers are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US region&lt;/b&gt;: Regional categories are defined by the Census Bureau: Midwest, Northeast, South, and West. Numbers for each region are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total of all counties in this Website&lt;/b&gt;: Numbers are presented for all counties included in this Website.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Counties&lt;/b&gt;: Previously chosen counties are listed. You can select a specific county for more detailed results. Detailed results include characteristics for each county (age, gender, payer, and race).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Number of discharges&lt;/b&gt;: The number of hospital stays (or discharges) for the selected condition or procedure is provided for each county.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Rate of discharges&lt;/b&gt;: The number of hospital stays (or discharges) divided by the number of residents in the county. County resident or population numbers are obtained from the US Census Bureau.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Mean costs in dollars&lt;/b&gt;: The mean or average cost is sometimes reported for each county. Costs are the actual value of these services (while charges are what a hospital asks to be paid for services). Total charges were converted to costs using &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; based on hospital accounting reports from the Centers for Medicare and Medicaid Services (CMS). In general, costs are less than charges.  AHRQ adjusts the &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; to work with this type of hospital data.&lt;/p&gt;	&lt;p&gt;If you select a specific county you will get following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Age group&lt;/b&gt;: Patient age in years is based on the admission date to the hospital and date of birth. The number of stays in each age group is reported.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Gender&lt;/b&gt;: The number of males and females is reported for the selected county and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Race&lt;/b&gt;: Race/ethnicity of the patient as listed in the medical record. Racial and ethnic categories are collapsed into larger groups based on US Census Bureau designations.&lt;br/&gt;&lt;br/&gt;You may notice some special codes in the tables:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Not enough data&lt;/b&gt;: When there are not enough data to report a value, a dash appears.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression&lt;/b&gt;: Small raw numbers may be suppressed to protect patient confidentiality. In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;</InterpretationText><ReportAttributes>DRGsDischargesFilters CountyFilters</ReportAttributes><IconSets /><Filters><Filter Type="ConditionsAndDiagnosis"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter><Filter Type="County"><Values><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns /></ReportManifest>'),('County Rates Detail Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="County Rates Detail Report" RptId="5AAF7FBA-7102-4C66-8598-A70597E2F826" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="CountyRatesDetail.PNG" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;P&gt;What are county rates of hospital use?&lt;/P&gt;	
	&lt;p&gt;Information is provided for each selected county by condition or procedure grouping. Select the titles in the top row to sort the results. Tables may include the following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US&lt;/b&gt;: National numbers are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US region&lt;/b&gt;: Regional categories are defined by the Census Bureau: Midwest, Northeast, South, and West. Numbers for each region are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total of all counties in this Website&lt;/b&gt;: Numbers are presented for all counties included in this Website.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Counties&lt;/b&gt;: Previously chosen counties are listed. You can select a specific county for more detailed results. Detailed results include characteristics for each county (age, gender, payer, and race).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Number of discharges&lt;/b&gt;: The number of hospital stays (or discharges) for the selected condition or procedure is provided for each county.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Rate of discharges&lt;/b&gt;: The number of hospital stays (or discharges) divided by the number of residents in the county. County resident or population numbers are obtained from the US Census Bureau.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Mean costs in dollars&lt;/b&gt;: The mean or average cost is sometimes reported for each county. Costs are the actual value of these services (while charges are what a hospital asks to be paid for services). Total charges were converted to costs using &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; based on hospital accounting reports from the Centers for Medicare and Medicaid Services (CMS). In general, costs are less than charges.  AHRQ adjusts the &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; to work with this type of hospital data.&lt;/p&gt;	&lt;p&gt;If you select a specific county you will get following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Age group&lt;/b&gt;: Patient age in years is based on the admission date to the hospital and date of birth. The number of stays in each age group is reported.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Gender&lt;/b&gt;: The number of males and females is reported for the selected county and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Race&lt;/b&gt;: Race/ethnicity of the patient as listed in the medical record. Racial and ethnic categories are collapsed into larger groups based on US Census Bureau designations.&lt;br/&gt;&lt;br/&gt;You may notice some special codes in the tables:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Not enough data&lt;/b&gt;: When there are not enough data to report a value, a dash appears.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression&lt;/b&gt;: Small raw numbers may be suppressed to protect patient confidentiality. In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;</InterpretationText><ReportAttributes>DRGsDischargesFilters CountyFilters</ReportAttributes><IconSets /><Filters><Filter Type="ConditionsAndDiagnosis"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter><Filter Type="County"><Values><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns /></ReportManifest>'),('County Rates Trending Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="County Rates Trending Report" RptId="26094800-7AF1-43FF-B476-66305C76C2F4" ShowInterpretationText="true" Category="Utilization" IsTrending="true" PreviewImage="CtyRatesDetailReport-Pop._TrendsReport - MQ60.jpg" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;P&gt;What are county rates of hospital use?&lt;/P&gt;	
	&lt;p&gt;Information is provided for each selected county by condition or procedure grouping. Select the titles in the top row to sort the results. Tables may include the following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US&lt;/b&gt;: National numbers are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US region&lt;/b&gt;: Regional categories are defined by the Census Bureau: Midwest, Northeast, South, and West. Numbers for each region are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total of all counties in this Website&lt;/b&gt;: Numbers are presented for all counties included in this Website.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Counties&lt;/b&gt;: Previously chosen counties are listed. You can select a specific county for more detailed results. Detailed results include characteristics for each county (age, gender, payer, and race).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Number of discharges&lt;/b&gt;: The number of hospital stays (or discharges) for the selected condition or procedure is provided for each county.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Rate of discharges&lt;/b&gt;: The number of hospital stays (or discharges) divided by the number of residents in the county. County resident or population numbers are obtained from the US Census Bureau.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Mean costs in dollars&lt;/b&gt;: The mean or average cost is sometimes reported for each county. Costs are the actual value of these services (while charges are what a hospital asks to be paid for services). Total charges were converted to costs using &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; based on hospital accounting reports from the Centers for Medicare and Medicaid Services (CMS). In general, costs are less than charges.  AHRQ adjusts the &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; to work with this type of hospital data.&lt;/p&gt;	&lt;p&gt;If you select a specific county you will get following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Age group&lt;/b&gt;: Patient age in years is based on the admission date to the hospital and date of birth. The number of stays in each age group is reported.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Gender&lt;/b&gt;: The number of males and females is reported for the selected county and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Race&lt;/b&gt;: Race/ethnicity of the patient as listed in the medical record. Racial and ethnic categories are collapsed into larger groups based on US Census Bureau designations.&lt;br/&gt;&lt;br/&gt;You may notice some special codes in the tables:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Not enough data&lt;/b&gt;: When there are not enough data to report a value, a dash appears.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression&lt;/b&gt;: Small raw numbers may be suppressed to protect patient confidentiality. In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;</InterpretationText><ReportAttributes>DRGsDischargesFilters CountyFilters</ReportAttributes><IconSets /><Filters><Filter Type="ConditionsAndDiagnosis"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter><Filter Type="County"><Values><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns /></ReportManifest>'),('ED Utilization Detailed Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="ED Utilization Detailed Report" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F828" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="EDUtilization-detail.png" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText> &lt;P&gt;How do I interpret the table?&lt;/p&gt;
	&lt;p&gt;Information is provided for each selected hospital by condition or procedure grouping.  Select the titles in the top row to sort the results.  Tables may include the following.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US: &lt;/b&gt;National numbers are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US region: &lt;/b&gt;Regional categories are defined by the Census Bureau: Midwest, Northeast, South, and West.  Numbers for each region are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ). &lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total of all hospitals in this Website: &lt;/b&gt;Numbers are presented for all hospitals included in this Website.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Hospitals: &lt;/b&gt;Previously chosen hospitals are listed.  You can select a hospital from the list for more detailed results.  Detailed results include characteristics of each hospital stay (age, gender, payer, and race).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Hospital County: &lt;/b&gt;The county for each hospital is provided.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# ED Visits:&lt;/b&gt; The total number of visits to the Emergency Department of the selected hospital.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# Admitted to Hospital Following ED Visit:&lt;/b&gt; The number of patients who visited the Emergency Department and were subsequently admitted to the Hospital.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# of Deaths During ED Visit:&lt;/b&gt; The number of patient deaths in the Emergency Department.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# Deaths Post-Admission Following ED Visit:&lt;/b&gt; The number of patient deaths post-admission to the hospital following a visit to the Emergency Department.&lt;br/&gt;&lt;br/&gt;If you select a specific hospital you will get following.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Age group: &lt;/b&gt;Patient age in years is based on the admission date to the hospital and date of birth.  The number of stays in each age is reported.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Gender: &lt;/b&gt;The number of males and females is reported for the selected hospital and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Payer: &lt;/b&gt;Payer is the expected payer for the hospital stay. Payers are grouped into general categories: Medicaid, Medicare, private insurance, uninsured, other, and missing.  The number of stays for each payer category is reported for the selected hospital and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Race: &lt;/b&gt;Race/ethnicity of the patient as listed in the medical record.  Racial and ethnic categories are collapsed into larger groups based on US Census Bureau designations.&lt;br/&gt;&lt;br/&gt;You may notice some special codes in the tables.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Dash (-): &lt;/b&gt;A dash is reported when there are not enough data for the given selection.  There are many reasons there may not be enough data to report.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression: &lt;/b&gt;Small raw numbers may be suppressed to protect patient confidentiality.  In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters ConditionsAndDiagnosisFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter><Filter Type="ConditionsAndDiagnosis"><Values><FilterValue Name="Health Condition or Topic" /></Values></Filter></Filters><Datasets><Dataset Name="ED Treat And Release" /></Datasets><Columns /></ReportManifest>'),('Emergency Department Treat-and-Release (ED) Utilization Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Emergency Department Treat-and-Release (ED) Utilization Report" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F827" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="EDUtilization.png" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText> &lt;P&gt;How do I interpret the table?&lt;/p&gt;
	&lt;p&gt;Information is provided for each selected hospital by condition or procedure grouping.  Select the titles in the top row to sort the results.  Tables may include the following.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US: &lt;/b&gt;National numbers are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US region: &lt;/b&gt;Regional categories are defined by the Census Bureau: Midwest, Northeast, South, and West.  Numbers for each region are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ). &lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total of all hospitals in this Website: &lt;/b&gt;Numbers are presented for all hospitals included in this Website.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Hospitals: &lt;/b&gt;Previously chosen hospitals are listed.  You can select a hospital from the list for more detailed results.  Detailed results include characteristics of each hospital stay (age, gender, payer, and race).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Hospital County: &lt;/b&gt;The county for each hospital is provided.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# ED Visits:&lt;/b&gt; The total number of visits to the Emergency Department of the selected hospital.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# Admitted to Hospital Following ED Visit:&lt;/b&gt; The number of patients who visited the Emergency Department and were subsequently admitted to the Hospital.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# of Deaths During ED Visit:&lt;/b&gt; The number of patient deaths in the Emergency Department.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# Deaths Post-Admission Following ED Visit:&lt;/b&gt; The number of patient deaths post-admission to the hospital following a visit to the Emergency Department.&lt;br/&gt;&lt;br/&gt;If you select a specific hospital you will get following.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Age group: &lt;/b&gt;Patient age in years is based on the admission date to the hospital and date of birth.  The number of stays in each age is reported.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Gender: &lt;/b&gt;The number of males and females is reported for the selected hospital and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Payer: &lt;/b&gt;Payer is the expected payer for the hospital stay. Payers are grouped into general categories: Medicaid, Medicare, private insurance, uninsured, other, and missing.  The number of stays for each payer category is reported for the selected hospital and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Race: &lt;/b&gt;Race/ethnicity of the patient as listed in the medical record.  Racial and ethnic categories are collapsed into larger groups based on US Census Bureau designations.&lt;br/&gt;&lt;br/&gt;You may notice some special codes in the tables.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Dash (-): &lt;/b&gt;A dash is reported when there are not enough data for the given selection.  There are many reasons there may not be enough data to report.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression: &lt;/b&gt;Small raw numbers may be suppressed to protect patient confidentiality.  In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters ConditionsAndDiagnosisFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter><Filter Type="ConditionsAndDiagnosis"><Values><FilterValue Name="Health Condition or Topic" /></Values></Filter></Filters><Datasets><Dataset Name="ED Treat And Release" /></Datasets><Columns /></ReportManifest>'),('ED Utilization detailed Report (stratified report)  Trending Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="ED Utilization detailed Report (stratified report)  Trending Report" RptId="4A2747DC-6B9D-4C77-963D-37B9B8112047" ShowInterpretationText="true" Category="Utilization" IsTrending="true" PreviewImage="ED_Util_DetailReport_TrendsReport - MQ60.jpg" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText /><ReportAttributes>HospitalFilters ConditionsAndDiagnosisFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter><Filter Type="ConditionsAndDiagnosis"><Values><FilterValue Name="Health Condition or Topic" /></Values></Filter></Filters><Datasets><Dataset Name="ED Treat And Release" /></Datasets><Columns /></ReportManifest>'),('Emergency Department Treat-and-Release (ED) Utilization Trending Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Emergency Department Treat-and-Release (ED) Utilization Trending Report" RptId="0D497B8D-E04E-4380-815D-7D9E644DC96E" ShowInterpretationText="true" Category="Utilization" IsTrending="true" PreviewImage="ED_Util_TreatRelease_TrendsReport - MQ60.jpg" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText> &lt;P&gt;How do I interpret the table?&lt;/p&gt;
	&lt;p&gt;Information is provided for each selected hospital by condition or procedure grouping.  Select the titles in the top row to sort the results.  Tables may include the following.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US: &lt;/b&gt;National numbers are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US region: &lt;/b&gt;Regional categories are defined by the Census Bureau: Midwest, Northeast, South, and West.  Numbers for each region are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ). &lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total of all hospitals in this Website: &lt;/b&gt;Numbers are presented for all hospitals included in this Website.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Hospitals: &lt;/b&gt;Previously chosen hospitals are listed.  You can select a hospital from the list for more detailed results.  Detailed results include characteristics of each hospital stay (age, gender, payer, and race).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Hospital County: &lt;/b&gt;The county for each hospital is provided.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# ED Visits:&lt;/b&gt; The total number of visits to the Emergency Department of the selected hospital.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# Admitted to Hospital Following ED Visit:&lt;/b&gt; The number of patients who visited the Emergency Department and were subsequently admitted to the Hospital.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# of Deaths During ED Visit:&lt;/b&gt; The number of patient deaths in the Emergency Department.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;# Deaths Post-Admission Following ED Visit:&lt;/b&gt; The number of patient deaths post-admission to the hospital following a visit to the Emergency Department.&lt;br/&gt;&lt;br/&gt;If you select a specific hospital you will get following.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Age group: &lt;/b&gt;Patient age in years is based on the admission date to the hospital and date of birth.  The number of stays in each age is reported.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Gender: &lt;/b&gt;The number of males and females is reported for the selected hospital and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Payer: &lt;/b&gt;Payer is the expected payer for the hospital stay. Payers are grouped into general categories: Medicaid, Medicare, private insurance, uninsured, other, and missing.  The number of stays for each payer category is reported for the selected hospital and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Race: &lt;/b&gt;Race/ethnicity of the patient as listed in the medical record.  Racial and ethnic categories are collapsed into larger groups based on US Census Bureau designations.&lt;br/&gt;&lt;br/&gt;You may notice some special codes in the tables.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Dash (-): &lt;/b&gt;A dash is reported when there are not enough data for the given selection.  There are many reasons there may not be enough data to report.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression: &lt;/b&gt;Small raw numbers may be suppressed to protect patient confidentiality.  In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters ConditionsAndDiagnosisFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter><Filter Type="ConditionsAndDiagnosis"><Values><FilterValue Name="Health Condition or Topic" /></Values></Filter></Filters><Datasets><Dataset Name="ED Treat And Release" /></Datasets><Columns /></ReportManifest>'),('Hospital Comparison Report - Bar Chart','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Hospital Comparison Report - Bar Chart" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F823" ShowInterpretationText="true" Category="Quality" IsTrending="false" PreviewImage="HospitalComparison-barchart.png" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="Consumers" /><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;How do I interpret the bar charts?&lt;/p&gt;
	&lt;p&gt;The bar charts show the specific scores used to create the hospital ratings. There is one bar for each hospital representing a measure in the comparison group.&lt;br/&gt;&lt;br/&gt;Depending on the type of rating, larger scores or smaller scores may be better.&lt;br/&gt;&lt;br/&gt;Each bar is labeled with the score.  In the detailed statistics, confidence intervals appear in parentheses.&lt;br/&gt;&lt;br/&gt;Each chart tells you how to interpret the length of the bars in that chart.&lt;br/&gt;&lt;br/&gt;&lt;ul style="margin-left:20px"&gt;&lt;b&gt;Figures presented are counts:&lt;/b&gt;&lt;br/&gt;Here the length of each bar shows the total number of patient stays.  For example, a total of 25 heart bypass surgeries were performed.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Figures presented are per 100 cases.&lt;/b&gt;&lt;br/&gt;Here the length of each bar shows the number of times an event occurred for every 100 hospital stays.  For example, for every 100 hospital stays for pneumonia, 5 of the patients died within thirty days.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Figures presented are per 1,000 cases.&lt;/b&gt;&lt;br/&gt;Here, the length of each bar shows the number of times an event occurred for every 1,000 hospital stays.  These are usually rarer events.  For example, for every 1,000 hospital stays for surgeries, 2 of the patients suffered breathing failure afterwards.&lt;/ul&gt;&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Hospital Compare Data" /><Dataset Name="AHRQ-QI Composite Data" /><Dataset Name="AHRQ-QI Provider Data" /></Datasets><Columns /></ReportManifest>'),('Hospital Comparison Report - Icons and Confidence Intervals','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Hospital Comparison Report - Icons and Confidence Intervals" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F820" ShowInterpretationText="true" Category="Quality" IsTrending="false" PreviewImage="HospitalComparison-icons-rates.png" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="Consumers" /><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;P&gt;What are hospital quality ratings?&lt;P&gt;		
	&lt;p&gt;There are many ways to judge the quality of health care.  Health care quality can be described as doing the right thing, at the right time, in the right way -- and having the best possible results.&lt;br/&gt;&lt;br/&gt;Quality ratings are based on specific quality indicators.  A quality indicator is a piece of information, usually a number, that shows how often patients had a particular experience when they received medical care. These experiences reflect a particular aspect of hospital quality.  This report uses three different types of quality indicators.&lt;br/&gt;&lt;br/&gt;&lt;a class="local-link" href="#/resources/AboutQualityRatings" &gt;Learn more about health care quality ratings.&lt;/a&gt;&lt;br/&gt;&lt;br/&gt;&lt;a href="#/resources/Methods" &gt;Learn more about hospital quality.&lt;/a&gt;&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters KeysForRatings</ReportAttributes><IconSets><IconSet IconType="IconSet1" /><IconSet IconType="IconSet2" /></IconSets><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Hospital Compare Data" /><Dataset Name="AHRQ-QI Composite Data" /><Dataset Name="AHRQ-QI Provider Data" /></Datasets><Columns /></ReportManifest>'),('Hospital Comparison Report - Icons','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Hospital Comparison Report - Icons" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F821" ShowInterpretationText="true" Category="Quality" IsTrending="false" PreviewImage="HospitalComparison-icons.png" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="Consumers" /><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;P&gt;What are hospital quality ratings?&lt;P&gt;		
	&lt;p&gt;There are many ways to judge the quality of health care.  Health care quality can be described as doing the right thing, at the right time, in the right way -- and having the best possible results.&lt;br/&gt;&lt;br/&gt;Quality ratings are based on specific quality indicators.  A quality indicator is a piece of information, usually a number, that shows how often patients had a particular experience when they received medical care. These experiences reflect a particular aspect of hospital quality.  This report uses three different types of quality indicators.&lt;br/&gt;&lt;br/&gt;&lt;a class="local-link" href="#/resources/AboutQualityRatings" &gt;Learn more about health care quality ratings.&lt;/a&gt;&lt;br/&gt;&lt;br/&gt;&lt;a href="#/resources/Methods" &gt;Learn more about hospital quality.&lt;/a&gt;&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters KeysForRatings</ReportAttributes><IconSets><IconSet IconType="IconSet1" /><IconSet IconType="IconSet2" /></IconSets><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Hospital Compare Data" /><Dataset Name="AHRQ-QI Composite Data" /><Dataset Name="AHRQ-QI Provider Data" /></Datasets><Columns /></ReportManifest>'),('Hospitals listings based on the search from the home page','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Hospitals listings based on the search from the home page" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F82B" ShowInterpretationText="false" Category="Quality" IsTrending="false" PreviewImage="HospitalListings.png" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="Consumers" /><Audience AudienceType="AllAudiences" /></Audiences><ReportAttributes>HospitalFilters KeysForRatings</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Hospital Compare Data" /></Datasets><Columns><Column Id="0" Name="Hospital Name" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Type of Hospital" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="How patients rate this hospital overall" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column></Columns></ReportManifest>'),('Hospital Profile Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Hospital Profile Report" RptId="7AF51434-5745-4538-B972-193F58E737D7" ShowInterpretationText="false" Category="Quality" IsTrending="false" PreviewImage="HospitalProfile.png" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="Consumers" /><Audience AudienceType="AllAudiences" /></Audiences><ReportAttributes>Display</ReportAttributes><IconSets /><Filters><Filter Type="Display"><Values><FilterValue Name="Basic Descriptive Data" /><FilterValue Name="Cost and Charge Data (All Patients)" Value="false" IsRadioButton="true" RadioGroupName="CostChargeDataGroup" /><FilterValue Name="Cost and Charge Data (Medicare)" IsRadioButton="true" RadioGroupName="CostChargeDataGroup" /><FilterValue Name="Map" /><FilterValue Name="Overall Patient Experience Ratings" /></Values></Filter></Filters><Datasets><Dataset Name="Hospital Compare Data" /><Dataset Name="Inpatient Discharge" /><Dataset Name="Medicare Provider Charge Data" /></Datasets><Columns /></ReportManifest>'),('Hospitals Quality Ratings - Bar Chart','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Hospitals Quality Ratings - Bar Chart" RptId="12546E5A-CA82-4D7E-AAB5-C1DB88A4FD33" ShowInterpretationText="true" Category="Quality" IsTrending="false" PreviewImage="HospitalsQualityRating-barchart.png" RequiresCmsProviderId="true" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="Consumers" /><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;How do I interpret the bar charts?&lt;/p&gt;
	&lt;p&gt;The bar charts show the specific scores used to create the hospital ratings. There is one bar for each hospital representing a measure in the comparison group.&lt;br/&gt;&lt;br/&gt;Depending on the type of rating, larger scores or smaller scores may be better.&lt;br/&gt;&lt;br/&gt;Each bar is labeled with the score.  In the detailed statistics, confidence intervals appear in parentheses.&lt;br/&gt;&lt;br/&gt;Each chart tells you how to interpret the length of the bars in that chart.&lt;br/&gt;&lt;br/&gt;&lt;ul style="margin-left:20px"&gt;&lt;b&gt;Figures presented are counts:&lt;/b&gt;&lt;br/&gt;Here the length of each bar shows the total number of patient stays.  For example, a total of 25 heart bypass surgeries were performed.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Figures presented are per 100 cases.&lt;/b&gt;&lt;br/&gt;Here the length of each bar shows the number of times an event occurred for every 100 hospital stays.  For example, for every 100 hospital stays for pneumonia, 5 of the patients died within thirty days.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Figures presented are per 1,000 cases.&lt;/b&gt;&lt;br/&gt;Here, the length of each bar shows the number of times an event occurred for every 1,000 hospital stays.  These are usually rarer events.  For example, for every 1,000 hospital stays for surgeries, 2 of the patients suffered breathing failure afterwards.&lt;/ul&gt;&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Hospital Compare Data" /><Dataset Name="AHRQ-QI Composite Data" /><Dataset Name="AHRQ-QI Provider Data" /></Datasets><Columns /></ReportManifest>'),('Hospitals Quality Ratings - Detail Tabular View','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Hospitals Quality Ratings - Detail Tabular View" RptId="7AAC8244-0F39-424A-85BE-943B465ED61B" ShowInterpretationText="true" Category="Quality" IsTrending="false" PreviewImage="HospitalsQualityRating-detail-tabularview.png" RequiresCmsProviderId="true" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;How do I interpret the detailed table?&lt;/P&gt;		
	&lt;p&gt;Different statistics are available for different quality indicators.  For more information on which statistics are available for each indicator, visit the &lt;a class="local-link" href="#/resources/AboutQualityRatings" &gt;Measure Details&lt;/a&gt; page. Numbers in the measure details table are not scaled.  These are raw statistics.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Numerator: &lt;/b&gt;The number of hospital stays in which the event or outcome of interest occurs (e.g., death caused by heart attack).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Denominator: &lt;/b&gt;The number of hospital stays in which the event or outcome of interest might potentially occur (e.g., admitted to the hospital for heart attack).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Observed rate: &lt;/b&gt;The numerator divided by the denominator. This rate is unadjusted.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Expected rate: &lt;/b&gt;The rate the hospital would have if it performed the same as the reference population given the hospital\&amp;#146;s actual case mix. In other words, the expected rate adjusts for differences in baseline characteristics (e.g., age, gender, DRG, and comorbidity categories) among patients admitted to given hospitals.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Risk adjustment: &lt;/b&gt;Risk adjustment is a statistical process that adjusts a hospital\&amp;#146;s estimated performance if the hospital had an "average" case mix.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Confidence interval: &lt;/b&gt;The confidence interval tells us we can be reasonably "confident" (in this case, 95% confident) that a hospital\&amp;#146;s rate fell somewhere within this specified range. The lower and upper bounds represent the lower and upper limits, respectively, of the confidence interval.  The smaller the range, the more precise the estimate.&lt;br/&gt;&lt;br/&gt;Information to help you interpret the statistics.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Ratio of observed/expected rate: &lt;/b&gt;Observed rate divided by expected rate.&lt;br/&gt;&lt;br/&gt;If the observed rate is greater than the expected rate (i.e., the ratio of observed/expected is greater than 1.0, or observed minus expected is positive), the hospital performed worse than the reference population for that particular indicator. Users may want to focus on these indicators for quality improvement.&lt;br/&gt;&lt;br/&gt;If the observed rate is less than the expected rate (i.e., the ratio of observed/expected is less than 1.0, or observed minus expected is negative), then the implication is that the hospital performed better than the reference population. Users may want to focus on these indicators for identifying best practices.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Number of patients: &lt;/b&gt;When hospitals treat a very large number of patients, chance differences will not have much effect on the overall rates. The range will be small and the estimated rates will be more precise. In hospitals that treat smaller numbers of patients, however, even small chance differences could have a big impact on rates. The 95% confidence interval will be large, and the estimated rates will be much less precise. Because the number of patients treated at U.S. hospitals varies widely, the precision of hospitals\&amp;#146; estimated rates also varies.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression: &lt;/b&gt;Small raw numbers may be suppressed to protect patient confidentiality. In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Hospital Compare Data" /><Dataset Name="AHRQ-QI Composite Data" /><Dataset Name="AHRQ-QI Provider Data" /></Datasets><Columns><Column Id="0" Name="Numerator" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Denominator" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Observed Rate" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Observed Lower-bound Confidence Interval (CI)" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Observed Upper-bound Confidence Interval (CI)" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Expected Rate" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Risk-Adjusted Rate" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Risk-Adjusted Lower-bound Confidence Interval (CI)" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Risk-Adjusted Upper-bound Confidence Interval (CI)" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column></Columns></ReportManifest>'),('Hospitals Quality Ratings - Icons with Rates and Confidence Intervals','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Hospitals Quality Ratings - Icons with Rates and Confidence Intervals" RptId="7AAC8244-0F39-424A-85BE-943B465ED61A" ShowInterpretationText="true" Category="Quality" IsTrending="false" PreviewImage="HospitalsQualityRating-icons-rates.png" RequiresCmsProviderId="true" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;P&gt;What are hospital quality ratings?&lt;P&gt;		
	&lt;p&gt;There are many ways to judge the quality of health care.  Health care quality can be described as doing the right thing, at the right time, in the right way -- and having the best possible results.&lt;br/&gt;&lt;br/&gt;Quality ratings are based on specific quality indicators.  A quality indicator is a piece of information, usually a number, that shows how often patients had a particular experience when they received medical care. These experiences reflect a particular aspect of hospital quality.  This report uses three different types of quality indicators.&lt;br/&gt;&lt;br/&gt;&lt;a class="local-link" href="#/resources/AboutQualityRatings" &gt;Learn more about health care quality ratings.&lt;/a&gt;&lt;br/&gt;&lt;br/&gt;&lt;a href="#/resources/Methods" &gt;Learn more about hospital quality.&lt;/a&gt;&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters KeysForRatings</ReportAttributes><IconSets><IconSet IconType="IconSet1" /><IconSet IconType="IconSet2" /></IconSets><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Hospital Compare Data" /><Dataset Name="AHRQ-QI Composite Data" /><Dataset Name="AHRQ-QI Provider Data" /></Datasets><Columns /></ReportManifest>'),('Hospitals Quality Ratings - Icons','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Hospitals Quality Ratings - Icons" RptId="1BD85413-734B-4C7A-9AAF-C442D8C2FACE" ShowInterpretationText="true" Category="Quality" IsTrending="false" PreviewImage="HospitalsQualityRating-icons.png" RequiresCmsProviderId="true" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="Consumers" /><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;P&gt;What are hospital quality ratings?&lt;P&gt;		
	&lt;p&gt;There are many ways to judge the quality of health care.  Health care quality can be described as doing the right thing, at the right time, in the right way -- and having the best possible results.&lt;br/&gt;&lt;br/&gt;Quality ratings are based on specific quality indicators.  A quality indicator is a piece of information, usually a number, that shows how often patients had a particular experience when they received medical care. These experiences reflect a particular aspect of hospital quality.  This report uses three different types of quality indicators.&lt;br/&gt;&lt;br/&gt;&lt;a class="local-link" href="#/resources/AboutQualityRatings" &gt;Learn more about health care quality ratings.&lt;/a&gt;&lt;br/&gt;&lt;br/&gt;&lt;a href="#/resources/Methods" &gt;Learn more about hospital quality.&lt;/a&gt;&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters KeysForRatings</ReportAttributes><IconSets><IconSet IconType="IconSet1" /><IconSet IconType="IconSet2" /></IconSets><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Hospital Compare Data" /><Dataset Name="AHRQ-QI Composite Data" /><Dataset Name="AHRQ-QI Provider Data" /></Datasets><Columns /></ReportManifest>'),('Inpatient Hospital Discharge Utilization Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Inpatient Hospital Discharge Utilization Report" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F824" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="IPHospitalDischargeUtilization.png" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;How do I interpret the table?&lt;/p&gt;
&lt;p&gt;Medicare Provider Charge Data (Inpatient). These include hospital-specific charges for the more than 3,000 U.S. hospitals that receive Medicare Inpatient Prospective Payment System (IPPS) payments for the top 100 most frequently billed discharges.&lt;/p&gt;
&lt;p&gt;Hospitals determine what they will charge for items and services provided to patients and these charges are the amount the hospital bills for an item or service. The Total Payment amount includes the MS-DRG amount, bill total per diem, beneficiary primary payer claim payment amount, beneficiary Part A coinsurance amount, beneficiary deductible amount, beneficiary blood deducible amount and DRG outlier amount.&lt;/p&gt;
&lt;p&gt;For these DRGs, average charges, average total payments, and average Medicare payments are calculated at the individual hospital level. Users will be able to make comparisons between the amount charged by individual hospitals within local markets, and nationwide, for services that might be furnished in connection with a particular inpatient stay.&lt;/p&gt;
&lt;p&gt;The latest data file can be downloaded from the MONAHRQ website under the &amp;ldquo;Resources&amp;rdquo; section. It is available in CSV format at http://www.cms.gov/Research-Statistics-Data-and-Systems/Statistics-Trends-and-Reports/Medicare-Provider-Charge-Data/Inpatient.html&lt;/p&gt;
&lt;p&gt;The downloaded file can be uploaded directly into MONAHRQ without making any adjustments.&lt;/p&gt;
&lt;p&gt;Information is provided for each selected hospital by Top 25 Diagnosis Related Groups (DRGs). Table includes the following.&lt;/p&gt;
&lt;p&gt;&lt;strong&gt;Number of discharges (principal):&lt;/strong&gt; The number of hospital stays (or discharges) for the selected DRG is provided for a hospital.&lt;/p&gt;
&lt;p&gt;&lt;strong&gt;Mean charges in dollars:&lt;/strong&gt; Hospitals determine what they will charge for items and services provided to patients and these charges are the amount the hospital bills for an item or service. This does not include professional (MD) fees. Charges are not necessarily how much was paid.&lt;/p&gt;
&lt;p&gt;&lt;strong&gt;Mean costs in dollars:&lt;/strong&gt; The average cost is sometimes reported for each hospital. Costs are the actual value of services performed (while charges represent the amount the hospital asked to be paid for services). Total charges were converted to costs using cost-to-charge ratios based on hospital accounting reports from the Centers for Medicare and Medicaid Services (CMS). In general, costs are less than charges. AHRQ adjusts the cost-to-charge ratios to work with this type of hospital data.&lt;/p&gt; 
&lt;p&gt;&lt;strong&gt;Mean Total Payments in dollars:&lt;/strong&gt; The Total Payment amount includes the MS-DRG amount, bill total per diem, beneficiary primary payer claim payment amount, beneficiary Part A coinsurance amount, beneficiary deductible amount, beneficiary blood deducible amount and DRG outlier amount.&lt;/p&gt;
</InterpretationText><ReportAttributes>HospitalFilters DRGsDischargesFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter><Filter Type="DRGsDischarges"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns><Column Id="0" Name="Number of discharges" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Mean charges in dollars" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Mean cost in dollars" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Median cost in dollars" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Mean length of stay in days" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column></Columns></ReportManifest>'),('Inpatient Hospital Discharge Utilization Trending Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Inpatient Hospital Discharge Utilization Trending Report" RptId="47426256-5F4F-4996-8F4A-3A344E39D90E" ShowInterpretationText="true" Category="Utilization" IsTrending="true" PreviewImage="IP_HospDischarge_Util_TrendsReport - MQ.60.jpg" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText> &lt;P&gt;How do I interpret the table?&lt;P&gt;
 &lt;p&gt;Information is provided for each selected hospital by condition or procedure grouping.  Select the titles in the top row to sort the results.  Tables may include the following.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US: &lt;/b&gt;National numbers are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2008, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US region: &lt;/b&gt;Regional categories are defined by the Census Bureau: Midwest, Northeast, South, and West.  Numbers for each region are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2008, Agency for Healthcare Research and Quality (AHRQ). &lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total of all hospitals in this Website: &lt;/b&gt;Numbers are presented for all hospitals included in this Website.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Hospitals: &lt;/b&gt;Previously chosen hospitals are listed.  You can select a hospital from the list for more detailed results.  Detailed results include characteristics of each hospital stay (age, gender, payer, and race).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Hospital County: &lt;/b&gt;The county for each hospital is provided.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Number of discharges (all-listed): &lt;/b&gt;The number of hospital stays (or discharges) for the selected condition or procedure is provided for each hospital.  All listed diagnoses include the principal diagnosis (or reason for going to the hospital) as well as any other conditions that coexist during the hospital stay.  All listed procedures include all procedures done for the patient.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Number of discharges (principal): &lt;/b&gt;The number of hospital stays (or discharges) for the selected condition or procedure is provided for each hospital.  Principal diagnosis means this is the condition chiefly responsible for admission to the hospital for care.  The principal procedure is the procedure that was done to address the principal diagnosis.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Mean charges in dollars: &lt;/b&gt;The mean or average charge is sometimes reported for each hospital.  This is the amount the hospital asked to be paid for services. This does not include professional (MD) fees.  Charges are not necessarily how much was paid.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Mean costs in dollars: &lt;/b&gt;The mean or average cost is sometimes reported for each hospital.  Costs are the actual value of services performed (while charges represent the amount the hospital asked to be paid for services). Total charges were converted to costs using &lt;a href=" http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge&lt;/a&gt; ratios based on hospital accounting reports from the Centers for Medicare and Medicaid Services (CMS). In general, costs are less than charges.  AHRQ adjusts the &lt;a href=" http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt;  to work with this type of hospital data.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Mean length of stay in days: &lt;/b&gt;The average or mean length of stay (LOS) is reported for each hospital.  This is the average number of nights the patient remained in the hospital. When a patient is admitted and discharged on the same day, it has a length of stay of zero.  A longer length of stay does not necessarily mean better or more care is given to patients.&lt;br/&gt;&lt;br/&gt;If you select a specific hospital you will get following.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Age group: &lt;/b&gt;Patient age in years is based on the admission date to the hospital and date of birth.  The number of stays in each age is reported.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Gender: &lt;/b&gt;The number of males and females is reported for the selected hospital and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Payer: &lt;/b&gt;Payer is the expected payer for the hospital stay. Payers are grouped into general categories: Medicaid, Medicare, private insurance, uninsured, other, and missing.  The number of stays for each payer category is reported for the selected hospital and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Race: &lt;/b&gt;Race/ethnicity of the patient as listed in the medical record.  Racial and ethnic categories are collapsed into larger groups based on US Census Bureau designations.&lt;br/&gt;&lt;br/&gt;You may notice some special codes in the tables.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Dash (-): &lt;/b&gt;A dash is reported when there are not enough data for the given selection.  There are many reasons there may not be enough data to report.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression: &lt;/b&gt;Small raw numbers may be suppressed to protect patient confidentiality.  In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;	
</InterpretationText><ReportAttributes>HospitalFilters DRGsDischargesFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter><Filter Type="DRGsDischarges"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns><Column Id="0" Name="Number of discharges" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Mean charges in dollars" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Mean cost in dollars" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Median cost in dollars" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Mean length of stay in days" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column></Columns></ReportManifest>'),('Inpatient Utilization Detail Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Inpatient Utilization Detail Report" RptId="2AAF7FBA-7102-4C66-8598-A70597E2F825" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="IPUtilizationDetail.png" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;How do I interpret the table?&lt;/p&gt;
&lt;p&gt;Medicare Provider Charge Data (Inpatient). These include hospital-specific charges for the more than 3,000 U.S. hospitals that receive Medicare Inpatient Prospective Payment System (IPPS) payments for the top 100 most frequently billed discharges.&lt;/p&gt;
&lt;p&gt;Hospitals determine what they will charge for items and services provided to patients and these charges are the amount the hospital bills for an item or service. The Total Payment amount includes the MS-DRG amount, bill total per diem, beneficiary primary payer claim payment amount, beneficiary Part A coinsurance amount, beneficiary deductible amount, beneficiary blood deducible amount and DRG outlier amount.&lt;/p&gt;
&lt;p&gt;For these DRGs, average charges, average total payments, and average Medicare payments are calculated at the individual hospital level. Users will be able to make comparisons between the amount charged by individual hospitals within local markets, and nationwide, for services that might be furnished in connection with a particular inpatient stay.&lt;/p&gt;
&lt;p&gt;The latest data file can be downloaded from the MONAHRQ website under the &amp;ldquo;Resources&amp;rdquo; section. It is available in CSV format at http://www.cms.gov/Research-Statistics-Data-and-Systems/Statistics-Trends-and-Reports/Medicare-Provider-Charge-Data/Inpatient.html&lt;/p&gt;
&lt;p&gt;The downloaded file can be uploaded directly into MONAHRQ without making any adjustments.&lt;/p&gt;
&lt;p&gt;Information is provided for each selected hospital by Top 25 Diagnosis Related Groups (DRGs). Table includes the following.&lt;/p&gt;
&lt;p&gt;&lt;strong&gt;Number of discharges (principal):&lt;/strong&gt; The number of hospital stays (or discharges) for the selected DRG is provided for a hospital.&lt;/p&gt;
&lt;p&gt;&lt;strong&gt;Mean charges in dollars:&lt;/strong&gt; Hospitals determine what they will charge for items and services provided to patients and these charges are the amount the hospital bills for an item or service. This does not include professional (MD) fees. Charges are not necessarily how much was paid.&lt;/p&gt;
&lt;p&gt;&lt;strong&gt;Mean costs in dollars:&lt;/strong&gt; The average cost is sometimes reported for each hospital. Costs are the actual value of services performed (while charges represent the amount the hospital asked to be paid for services). Total charges were converted to costs using cost-to-charge ratios based on hospital accounting reports from the Centers for Medicare and Medicaid Services (CMS). In general, costs are less than charges. AHRQ adjusts the cost-to-charge ratios to work with this type of hospital data.&lt;/p&gt; 
&lt;p&gt;&lt;strong&gt;Mean Total Payments in dollars:&lt;/strong&gt; The Total Payment amount includes the MS-DRG amount, bill total per diem, beneficiary primary payer claim payment amount, beneficiary Part A coinsurance amount, beneficiary deductible amount, beneficiary blood deducible amount and DRG outlier amount.&lt;/p&gt;
</InterpretationText><ReportAttributes>HospitalFilters DRGsDischargesFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter><Filter Type="DRGsDischarges"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns /></ReportManifest>'),('IP Utilization detailed Report (stratified report)  Trending Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="IP Utilization detailed Report (stratified report)  Trending Report" RptId="4E94E281-8E7E-4B02-8567-72B4FA239F9E" ShowInterpretationText="true" Category="Utilization" IsTrending="true" PreviewImage="IP_Util_DetailedReport_StratReport _TrendsReport - MQ.60.jpg" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText /><ReportAttributes>HospitalFilters DRGsDischargesFilters</ReportAttributes><IconSets /><Filters><Filter Type="Hospital"><Values><FilterValue Name="Hospital Name" /><FilterValue Name="Hospital Type" /><FilterValue Name="Region" /><FilterValue Name="County" /></Values></Filter><Filter Type="DRGsDischarges"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns /></ReportManifest>'),('Nursing home comparison report (up to five nursing homes) with individual measure scores by domains','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Nursing home comparison report (up to five nursing homes) with individual measure scores by domains" RptId="F2F2B7FE-8653-488B-8ED8-FD4417CD0F9E" ShowInterpretationText="true" Category="Quality" IsTrending="false" PreviewImage="NursingHome_Compare_Indiv_Meas - MQ60.jpg" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;&lt;strong&gt;How to interpret content&lt;/strong&gt;
    &lt;br&gt;Each rating in the report is shown as a simple image along with a word or two. The words will tell you how a particular nursing home compares to the average rating of the other homes in the area. Going from best to worst, the rating will show that a nursing home is:
    &lt;ul&gt;
        &lt;li&gt;&lt;span class="ng-scope ng-isolate-scope"&gt;&lt;span class="rating-nursing"&gt;
  &lt;img width="155" height="24" src="themes/base/assets/stars_5.png" class=""&gt;&lt;br&gt;
  &lt;span class="rating-nursing--title ng-binding"&gt;Much better than average&lt;/span&gt;
  &lt;span class="rating-nursing--subtitle ng-binding"&gt; &lt;/span&gt;
&lt;/span&gt;&lt;/li&gt;
        &lt;li&gt;&lt;span class="ng-scope ng-isolate-scope"&gt;&lt;span class="rating-nursing"&gt;
  &lt;img  height="24" src="themes/base/assets/stars_4.png" class=""&gt;&lt;br&gt;
  &lt;span class="rating-nursing--title ng-binding"&gt;Better than average&lt;/span&gt;
  &lt;span class="rating-nursing--subtitle ng-binding"&gt; &lt;/span&gt;
&lt;/span&gt;&lt;/li&gt;
        &lt;li&gt;&lt;span class="ng-scope ng-isolate-scope"&gt;&lt;span class="rating-nursing"&gt;
  &lt;img width="155" height="24" src="themes/base/assets/stars_3.png" class=""&gt;&lt;br&gt;
  &lt;span class="rating-nursing--title ng-binding"&gt;Average&lt;/span&gt;
  &lt;span class="rating-nursing--subtitle ng-binding"&gt; &lt;/span&gt;
&lt;/span&gt;&lt;/li&gt;
        &lt;li&gt;&lt;span class="ng-scope ng-isolate-scope"&gt;&lt;span class="rating-nursing"&gt;
  &lt;img  width="155" height="24" src="themes/base/assets/stars_2.png" class=""&gt;&lt;br&gt;
  &lt;span class="rating-nursing--title ng-binding"&gt;Below average&lt;/span&gt;
  &lt;span class="rating-nursing--subtitle ng-binding"&gt; &lt;/span&gt;
&lt;/span&gt;&lt;/li&gt;
        &lt;li&gt;&lt;span class="ng-scope ng-isolate-scope"&gt;&lt;span class="rating-nursing"&gt;
  &lt;img width="155" height="24" src="themes/base/assets/stars_1.png" class=""&gt;&lt;br&gt;
  &lt;span class="rating-nursing--title ng-binding"&gt;Much below average&lt;/span&gt;
  &lt;span class="rating-nursing--subtitle ng-binding"&gt; &lt;/span&gt;
&lt;/span&gt;&lt;/li&gt;
    &lt;/ul&gt;
    &lt;br&gt;&lt;strong&gt;The information on this website comes from two sources:&lt;/strong&gt;
    &lt;br&gt;1. Yearly health and safety inspections of each nursing home that are done by state agencies. Health and safety information, and staffing information, come from reports on these inspections.
    &lt;br&gt;2. Monthly information that each nursing home submits to the federal government about each resident. Quality information comes from this information source.
    &lt;br&gt;
    &lt;br&gt;&lt;strong&gt;Whats here?&lt;/strong&gt;
    &lt;br&gt;There are three types of information in this Report:
    &lt;ul&gt;
        &lt;li&gt;&lt;strong&gt;Quality Information&lt;/strong&gt;, such as whether residents are receiving appropriate care what is happening to their health and ability to function&lt;/li&gt;
        &lt;li&gt;&lt;strong&gt;Health and Safety Information&lt;/strong&gt;, such as number of health and safety problems found when state agencies did an inspection&lt;/li&gt;
        &lt;li&gt;&lt;strong&gt;Staffing Information&lt;/strong&gt;, such as the number of nursing home staff available to care for residents during their stay&lt;/li&gt;
    &lt;/ul&gt;
    &lt;br&gt;This information helps you compare nursing homes to each other and see which are better. You can compare up to 5 nursing homes in terms of their Overall Score, their Inspection Score, their Quality Score and their Staffing Score.
    &lt;br&gt;
    &lt;br&gt;&lt;strong&gt;How can I use this information?&lt;/strong&gt;
    &lt;br&gt;You can use this information to:
    &lt;ul&gt;
        &lt;li&gt;help you choose a nursing home for yourself&lt;/li&gt;
        &lt;li&gt;provide useful information for your loved ones if they need nursing home care&lt;/li&gt;
        &lt;li&gt;encourage nursing homes to improve their quality&lt;/li&gt;
    &lt;/ul&gt;
&lt;/p&gt;</InterpretationText><ReportAttributes>None</ReportAttributes><IconSets /><Filters /><Datasets><Dataset Name="Nursing Home Compare Data" /></Datasets><Columns /></ReportManifest>'),('Nursing home profile report with summary and individual measure score by domain','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Nursing home profile report with summary and individual measure score by domain" RptId="87E04110-46B0-4CAE-9592-022C3111FAC7" ShowInterpretationText="true" Category="Quality" IsTrending="false" PreviewImage="NursingHome_ProfileReport_Summary - MQ60.jpg" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;&lt;strong style=line-height:1.6&gt;Ownership Type:&lt;/strong&gt;&lt;span style=line-height:1.6&gt;&amp;nbsp; Nursing homes can be owned by private for-profit corporations, non-profit organizations, religious affiliated organizations or government agencies.&lt;/span&gt;&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Number of Certified Beds:&lt;/strong&gt;&amp;nbsp; Certified beds are beds in the nursing home that have been approved by the federal government.&amp;nbsp; Larger nursing homes have more beds.&amp;nbsp;&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Number of Residents in Certified Beds:&lt;/strong&gt; Certified beds are beds in the nursing home that have been approved by the federal government.&amp;nbsp; When a nursing home has fewer residents in beds than the number of certified beds, they should have room for new residents.&amp;nbsp;&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Provider Type:&lt;/strong&gt; Whether the nursing home participates in the federal Medicare program, the Medicaid program, or both.&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Is the nursing home located in a hospital?&lt;/strong&gt;&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Is the nursing home located in a continuing care retirement community?&lt;/strong&gt; A continuing care retirement community is a place that offers different levels of care for older people, including independent living, assisted living, and skilled nursing.&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Date first approved to provide Medicare and Medicaid services&lt;/strong&gt;&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Special Focus Facility:&lt;/strong&gt; Special Focus Facilities (SFF) are nursing homes that have:&lt;/p&gt;&lt;ul&gt;&lt;li&gt;&lt;strong&gt;More health and safety problems&lt;/strong&gt; than other nursing homes (about twice the average number of deficiencies),&lt;/li&gt;&lt;li&gt;&lt;strong&gt;More serious problems&lt;/strong&gt; than most other nursing homes (including harm or injury experienced by residents), and&lt;/li&gt;&lt;li&gt;&lt;strong&gt;A pattern of serious problems&lt;/strong&gt;that has continued &lt;strong&gt;over a long time&lt;/strong&gt;&lt;/li&gt;&lt;/ul&gt;&lt;p&gt;The federal government requires that Special Focus nursing homes be inspected twice as often&amp;nbsp; as other nursing homes (about twice per year). These visits are used to see whether the nursing home has improved the identified problems or is in need of disciplinary action.&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Does the nursing home have a resident and family council?&lt;/strong&gt;Resident and family councils can help residents and their families communicate with nursing home staff. The law requires nursing homes to allow councils to be set up by residents and families.&lt;/p&gt;&lt;p&gt;&lt;strong style=line-height:1.6&gt;Accreditation:&lt;/strong&gt;&lt;span style=line-height:1.6&gt;&amp;nbsp; Nursing homes can choose to be &amp;ldquo;accredited&amp;rdquo; by one or more organization that determines if a given facility meets its standards.&amp;nbsp; Some very good nursing homes, however, do not choose to be accredited. &amp;nbsp;&lt;/span&gt;&lt;/p&gt;</InterpretationText><ReportAttributes>None</ReportAttributes><IconSets /><Filters><Filter Type="Display"><Values><FilterValue Name="Basic Descriptive Data" /><FilterValue Name="Map" /><FilterValue Name="Overall Score" /><FilterValue Name="Quality Measures" /><FilterValue Name="Health Inspection" /><FilterValue Name="Staffing" /></Values></Filter></Filters><Datasets><Dataset Name="Nursing Home Compare Data" /></Datasets><Columns /><Footer>The information on this website should be interpreted cautiously as there are both strengths and weaknesses associated with the data. You may want to use information from many different sources to learn about a nursing homes quality, such as the long term care ombudsman''s office in your state or the state agency that inspects nursing homes. Many experts strongly recommend visiting a nursing home in person before deciding to choose it for yourself or your loved one.</Footer></ReportManifest>'),('Find nursing home with overall and domain level summary ratings report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Find nursing home with overall and domain level summary ratings report" RptId="BA52B7B2-F4C8-4831-B910-1D036B94AE75" ShowInterpretationText="true" Category="Quality" IsTrending="false" PreviewImage="CountyRatesTrending.PNG" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;&lt;strong&gt;How to interpret content&lt;/strong&gt;
    &lt;br&gt;Each rating in the report is shown as a simple image along with a word or two. The words will tell you how a particular nursing home compares to the average rating of the other homes in the area. Going from best to worst, the rating will show that a nursing home is:
    &lt;ul&gt;
        &lt;li&gt;&lt;span class="ng-scope ng-isolate-scope"&gt;&lt;span class="rating-nursing"&gt;
  &lt;img width="155" height="24" src="themes/base/assets/stars_5.png" class=""&gt;&lt;br&gt;
  &lt;span class="rating-nursing--title ng-binding"&gt;Much better than average&lt;/span&gt;
  &lt;span class="rating-nursing--subtitle ng-binding"&gt; &lt;/span&gt;
&lt;/span&gt;&lt;/li&gt;
        &lt;li&gt;&lt;span class="ng-scope ng-isolate-scope"&gt;&lt;span class="rating-nursing"&gt;
  &lt;img  height="24" src="themes/base/assets/stars_4.png" class=""&gt;&lt;br&gt;
  &lt;span class="rating-nursing--title ng-binding"&gt;Better than average&lt;/span&gt;
  &lt;span class="rating-nursing--subtitle ng-binding"&gt; &lt;/span&gt;
&lt;/span&gt;&lt;/li&gt;
        &lt;li&gt;&lt;span class="ng-scope ng-isolate-scope"&gt;&lt;span class="rating-nursing"&gt;
  &lt;img width="155" height="24" src="themes/base/assets/stars_3.png" class=""&gt;&lt;br&gt;
  &lt;span class="rating-nursing--title ng-binding"&gt;Average&lt;/span&gt;
  &lt;span class="rating-nursing--subtitle ng-binding"&gt; &lt;/span&gt;
&lt;/span&gt;&lt;/li&gt;
        &lt;li&gt;&lt;span class="ng-scope ng-isolate-scope"&gt;&lt;span class="rating-nursing"&gt;
  &lt;img  width="155" height="24" src="themes/base/assets/stars_2.png" class=""&gt;&lt;br&gt;
  &lt;span class="rating-nursing--title ng-binding"&gt;Below average&lt;/span&gt;
  &lt;span class="rating-nursing--subtitle ng-binding"&gt; &lt;/span&gt;
&lt;/span&gt;&lt;/li&gt;
        &lt;li&gt;&lt;span class="ng-scope ng-isolate-scope"&gt;&lt;span class="rating-nursing"&gt;
  &lt;img width="155" height="24" src="themes/base/assets/stars_1.png" class=""&gt;&lt;br&gt;
  &lt;span class="rating-nursing--title ng-binding"&gt;Much below average&lt;/span&gt;
  &lt;span class="rating-nursing--subtitle ng-binding"&gt; &lt;/span&gt;
&lt;/span&gt;&lt;/li&gt;
    &lt;/ul&gt;
    &lt;br&gt;&lt;strong&gt;The information on this website comes from two sources:&lt;/strong&gt;
    &lt;br&gt;1. Yearly health and safety inspections of each nursing home that are done by state agencies. Health and safety information, and staffing information, come from reports on these inspections.
    &lt;br&gt;2. Monthly information that each nursing home submits to the federal government about each resident. Quality information comes from this information source.
    &lt;br&gt;
    &lt;br&gt;&lt;strong&gt;Whats here?&lt;/strong&gt;
    &lt;br&gt;There are three types of information in this Report:
    &lt;ul&gt;
        &lt;li&gt;&lt;strong&gt;Quality Information&lt;/strong&gt;, such as whether residents are receiving appropriate care what is happening to their health and ability to function&lt;/li&gt;
        &lt;li&gt;&lt;strong&gt;Health and Safety Information&lt;/strong&gt;, such as number of health and safety problems found when state agencies did an inspection&lt;/li&gt;
        &lt;li&gt;&lt;strong&gt;Staffing Information&lt;/strong&gt;, such as the number of nursing home staff available to care for residents during their stay&lt;/li&gt;
    &lt;/ul&gt;
    &lt;br&gt;This information helps you compare nursing homes to each other and see which are better. You can compare up to 5 nursing homes in terms of their Overall Score, their Inspection Score, their Quality Score and their Staffing Score.
    &lt;br&gt;
    &lt;br&gt;&lt;strong&gt;How can I use this information?&lt;/strong&gt;
    &lt;br&gt;You can use this information to:
    &lt;ul&gt;
        &lt;li&gt;help you choose a nursing home for yourself&lt;/li&gt;
        &lt;li&gt;provide useful information for your loved ones if they need nursing home care&lt;/li&gt;
        &lt;li&gt;encourage nursing homes to improve their quality&lt;/li&gt;
    &lt;/ul&gt;
&lt;/p&gt;</InterpretationText><ReportAttributes>None</ReportAttributes><IconSets /><Filters><Filter Type="NursingHomeFilters"><Values><FilterValue Name="Nursing Home Name" /><FilterValue Name="Nursing Home Type (Medicare/Medicaid/Both)" /><FilterValue Name="In Hospital" /></Values></Filter><Filter Type="GeoLocation"><Values><FilterValue Name="County" /><FilterValue Name="Zip" /></Values></Filter><Filter Type="Display"><Values><FilterValue Name="Show on Map" /></Values></Filter></Filters><Datasets><Dataset Name="Nursing Home Compare Data" /></Datasets><Columns /></ReportManifest>'),('Physician Listing Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Physician Listing Report" RptId="E007BB9C-E539-41D6-9D06-FF52F8A15BF6" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="Physician_FindPhysicianReport - MQ60.jpg" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText /><ReportAttributes>None</ReportAttributes><IconSets /><Filters><Filter Type="PhysicianFilters"><Values><FilterValue Name="Physician Name" /><FilterValue Name="Practice Name" /><FilterValue Name="Geo Location" /><FilterValue Name="Zip" /><FilterValue Name="Specialty" /></Values></Filter></Filters><Datasets><Dataset Name="Physician Data" /></Datasets><Columns /></ReportManifest>'),('Physician Profile','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Physician Profile" RptId="4C5727B4-0E85-4F80-ADE9-418B49A1373E" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="PhysicianProfile - MQ60.jpg" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;p&gt;&lt;strong&gt;ASSIGNMENT/PARTICIPATION&lt;/strong&gt;&amp;nbsp;&lt;/p&gt;&lt;p&gt;&lt;strong&gt;&lt;em&gt;Accepts Medicare Assignment:&lt;/em&gt;&lt;/strong&gt;&amp;nbsp; If Yes, then this physician is willing to accept the Medicare allowable charge for his/her services as payment in full.&amp;nbsp; If No, then you will be responsible for paying what the physicians charges, even if it is higher than the Medicare allowable charge.&amp;nbsp;&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Participating in eRx:&lt;/strong&gt;&amp;nbsp; This program has ended so we should not include this.&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Participating in PQRS:&lt;/strong&gt;&amp;nbsp; If Yes, then this physician participates in Medicare&amp;rsquo;s Physician Quality Reporting System.&amp;nbsp; This program encourages physicians to report information on the quality of care they provide to patients to the Medicare program through financial incentives.&amp;nbsp; If No, then s/he does not.&amp;nbsp;&lt;/p&gt;&lt;p&gt;&lt;strong&gt;Participating in EHR:&lt;/strong&gt;&amp;nbsp;If Yes, then this physicians participates in Medicare&amp;rsquo;s Electronic Health Record program.&amp;nbsp; This program provides financial support to physicians to start or improve electronic systems to maintain information on patients and their care that can be used to improve care.&lt;/p&gt;</InterpretationText><ReportAttributes>Display</ReportAttributes><IconSets /><Filters><Filter Type="Display"><Values><FilterValue Name="Basic Descriptive Data" /><FilterValue Name="Map" /><FilterValue Name="Affiliated Hospitals" /></Values></Filter></Filters><Datasets><Dataset Name="Physician Data" /></Datasets><Columns /></ReportManifest>'),('Region Rates Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Region Rates Report" RptId="3a40cf6b-37ad-4861-b272-930ddf2b8802" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="RegionRates.PNG" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;P&gt;What are region rates of hospital use?&lt;/P&gt;	
	&lt;p&gt;Information is provided for each selected region by condition or procedure grouping. Select the titles in the top row to sort the results. Tables may include the following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US&lt;/b&gt;: National numbers are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US region&lt;/b&gt;: Regional categories are defined by the Census Bureau: Midwest, Northeast, South, and West. Numbers for each region are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total of all counties in this Website&lt;/b&gt;: Numbers are presented for all counties included in this Website.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Counties&lt;/b&gt;: Previously chosen counties are listed. You can select a specific region for more detailed results. Detailed results include characteristics for each region (age, gender, payer, and race).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Number of discharges&lt;/b&gt;: The number of hospital stays (or discharges) for the selected condition or procedure is provided for each region.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Rate of discharges&lt;/b&gt;: The number of hospital stays (or discharges) divided by the number of residents in the region. region resident or population numbers are obtained from the US Census Bureau.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Mean costs in dollars&lt;/b&gt;: The mean or average cost is sometimes reported for each region. Costs are the actual value of these services (while charges are what a hospital asks to be paid for services). Total charges were converted to costs using &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; based on hospital accounting reports from the Centers for Medicare and Medicaid Services (CMS). In general, costs are less than charges.  AHRQ adjusts the &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; to work with this type of hospital data.&lt;/p&gt;	&lt;p&gt;If you select a specific region you will get following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Age group&lt;/b&gt;: Patient age in years is based on the admission date to the hospital and date of birth. The number of stays in each age group is reported.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Gender&lt;/b&gt;: The number of males and females is reported for the selected region and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Race&lt;/b&gt;: Race/ethnicity of the patient as listed in the medical record. Racial and ethnic categories are collapsed into larger groups based on US Census Bureau designations.&lt;br/&gt;&lt;br/&gt;You may notice some special codes in the tables:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Not enough data&lt;/b&gt;: When there are not enough data to report a value, a dash appears.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression&lt;/b&gt;: Small raw numbers may be suppressed to protect patient confidentiality. In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;</InterpretationText><ReportAttributes>DRGsDischargesFilters</ReportAttributes><IconSets /><Filters><Filter Type="DRGsDischarges"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns><Column Id="0" Name="Number of discharges" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Mean cost in dollars" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Rate of Discharges Per 1000 Persons" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column></Columns></ReportManifest>'),('Region Rates Detail Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Region Rates Detail Report" RptId="8d25f78e-86ba-43fb-ba5f-352227187759" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="RegionRatesDetail.PNG" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;P&gt;What are region rates of hospital use?&lt;/P&gt;	
	&lt;p&gt;Information is provided for each selected region by condition or procedure grouping. Select the titles in the top row to sort the results. Tables may include the following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US&lt;/b&gt;: National numbers are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US region&lt;/b&gt;: Regional categories are defined by the Census Bureau: Midwest, Northeast, South, and West. Numbers for each region are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total of all counties in this Website&lt;/b&gt;: Numbers are presented for all counties included in this Website.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Counties&lt;/b&gt;: Previously chosen counties are listed. You can select a specific region for more detailed results. Detailed results include characteristics for each region (age, gender, payer, and race).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Number of discharges&lt;/b&gt;: The number of hospital stays (or discharges) for the selected condition or procedure is provided for each region.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Rate of discharges&lt;/b&gt;: The number of hospital stays (or discharges) divided by the number of residents in the region. region resident or population numbers are obtained from the US Census Bureau.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Mean costs in dollars&lt;/b&gt;: The mean or average cost is sometimes reported for each region. Costs are the actual value of these services (while charges are what a hospital asks to be paid for services). Total charges were converted to costs using &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; based on hospital accounting reports from the Centers for Medicare and Medicaid Services (CMS). In general, costs are less than charges.  AHRQ adjusts the &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; to work with this type of hospital data.&lt;/p&gt;	&lt;p&gt;If you select a specific region you will get following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Age group&lt;/b&gt;: Patient age in years is based on the admission date to the hospital and date of birth. The number of stays in each age group is reported.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Gender&lt;/b&gt;: The number of males and females is reported for the selected region and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Race&lt;/b&gt;: Race/ethnicity of the patient as listed in the medical record. Racial and ethnic categories are collapsed into larger groups based on US Census Bureau designations.&lt;br/&gt;&lt;br/&gt;You may notice some special codes in the tables:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Not enough data&lt;/b&gt;: When there are not enough data to report a value, a dash appears.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression&lt;/b&gt;: Small raw numbers may be suppressed to protect patient confidentiality. In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;</InterpretationText><ReportAttributes>DRGsDischargesFilters</ReportAttributes><IconSets /><Filters><Filter Type="DRGsDischarges"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns><Column Id="0" Name="Number of discharges" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Mean cost in dollars" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Rate of Discharges Per [x] Persons" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column></Columns></ReportManifest>'),('Region Rates Trending Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Region Rates Trending Report" RptId="709F70B5-50D9-4CA9-A19D-9E3DF2B38386" ShowInterpretationText="true" Category="Utilization" IsTrending="true" PreviewImage="Reg_Rates_Report-Pop._Trends Report - MQ60.jpg" RequiresCmsProviderId="false" RequiresCostToChargeRatio="true"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText>&lt;P&gt;What are region rates of hospital use?&lt;/P&gt;	
	&lt;p&gt;Information is provided for each selected region by condition or procedure grouping. Select the titles in the top row to sort the results. Tables may include the following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US&lt;/b&gt;: National numbers are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total US region&lt;/b&gt;: Regional categories are defined by the Census Bureau: Midwest, Northeast, South, and West. Numbers for each region are weighted estimates from the &lt;a href="http://www.hcup-us.ahrq.gov/" target="_blank"&gt;HCUP Nationwide Inpatient Sample&lt;/a&gt; (NIS), 2011, Agency for Healthcare Research and Quality (AHRQ).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Total of all counties in this Website&lt;/b&gt;: Numbers are presented for all counties included in this Website.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Counties&lt;/b&gt;: Previously chosen counties are listed. You can select a specific region for more detailed results. Detailed results include characteristics for each region (age, gender, payer, and race).&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Number of discharges&lt;/b&gt;: The number of hospital stays (or discharges) for the selected condition or procedure is provided for each region.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Rate of discharges&lt;/b&gt;: The number of hospital stays (or discharges) divided by the number of residents in the region. region resident or population numbers are obtained from the US Census Bureau.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Mean costs in dollars&lt;/b&gt;: The mean or average cost is sometimes reported for each region. Costs are the actual value of these services (while charges are what a hospital asks to be paid for services). Total charges were converted to costs using &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; based on hospital accounting reports from the Centers for Medicare and Medicaid Services (CMS). In general, costs are less than charges.  AHRQ adjusts the &lt;a href="http://www.hcup-us.ahrq.gov/db/state/costtocharge.jsp" target="_blank"&gt;cost-to-charge ratios&lt;/a&gt; to work with this type of hospital data.&lt;/p&gt;	&lt;p&gt;If you select a specific region you will get following:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Age group&lt;/b&gt;: Patient age in years is based on the admission date to the hospital and date of birth. The number of stays in each age group is reported.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Gender&lt;/b&gt;: The number of males and females is reported for the selected region and condition or procedure.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Race&lt;/b&gt;: Race/ethnicity of the patient as listed in the medical record. Racial and ethnic categories are collapsed into larger groups based on US Census Bureau designations.&lt;br/&gt;&lt;br/&gt;You may notice some special codes in the tables:&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Not enough data&lt;/b&gt;: When there are not enough data to report a value, a dash appears.&lt;br/&gt;&lt;br/&gt;&lt;b&gt;Small number suppression&lt;/b&gt;: Small raw numbers may be suppressed to protect patient confidentiality. In this case, a &amp;quot;c&amp;quot; appears.&lt;/p&gt;</InterpretationText><ReportAttributes>DRGsDischargesFilters</ReportAttributes><IconSets /><Filters><Filter Type="DRGsDischarges"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns><Column Id="0" Name="Number of discharges" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Mean cost in dollars" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Rate of Discharges Per 1000 Persons" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column></Columns></ReportManifest>'),('Infographics Report -  Surgical Safety','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Infographics Report -  Surgical Safety" RptId="64B545BF-D183-4F0D-8ABC-6643FF39F5DD" ShowInterpretationText="true" Category="Utilization" IsTrending="false" PreviewImage="CountyRatesTrending.PNG" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText /><ReportAttributes>None</ReportAttributes><IconSets /><Filters><Filter Type="ActiveSections"><Values><FilterValue Name="CMS Measures" /><FilterValue Name="AHRQ QI Measures" /></Values></Filter></Filters><Datasets><Dataset Name="AHRQ-QI Area Data" /><Dataset Name="Hospital Compare Data" /></Datasets><Columns /></ReportManifest>'),('Utilization Rates by County, Detail Report (Population)  Trending Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Utilization Rates by County, Detail Report (Population)  Trending Report" RptId="412A47D6-0859-47C7-A980-0A6C395C55F4" ShowInterpretationText="true" Category="Utilization" IsTrending="true" PreviewImage="CountyRatesTrending.PNG" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText /><ReportAttributes>DRGsDischargesFilters CountyFilters</ReportAttributes><IconSets /><Filters><Filter Type="ConditionsAndDiagnosis"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter><Filter Type="County"><Values><FilterValue Name="County" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns /></ReportManifest>'),('Utilization Rates by Region, Detail Report (Population)  Trending Report','<ReportManifest xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" Id="0" Name="Utilization Rates by Region, Detail Report (Population)  Trending Report" RptId="9D45AFA3-2D1C-48E6-9B9F-77746DF75C63" ShowInterpretationText="true" Category="Utilization" IsTrending="true" PreviewImage="CountyRatesTrending.PNG" RequiresCmsProviderId="false" RequiresCostToChargeRatio="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit><Audiences><Audience AudienceType="AllAudiences" /></Audiences><InterpretationText /><ReportAttributes>DRGsDischargesFilters</ReportAttributes><IconSets /><Filters><Filter Type="DRGsDischarges"><Values><FilterValue Name="Major Diagnosis Category" /><FilterValue Name="Diagnosis Related Group" /><FilterValue Name="Health Condition or Topic" /><FilterValue Name="Procedure" /></Values></Filter></Filters><Datasets><Dataset Name="Inpatient Discharge" /></Datasets><Columns><Column Id="0" Name="Number of discharges" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Mean cost in dollars" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column><Column Id="0" Name="Rate of Discharges Per [x] Persons" IsMeasure="false"><IsChanged>true</IsChanged><IsDeleted>false</IsDeleted><SkipAudit>false</SkipAudit></Column></Columns></ReportManifest>')) 
 AS Val (Col1, Col2) on r.ReportType = Val.Col1
WHEN MATCHED THEN 
	 UPDATE SET r.SourceTemplateXml = Val.Col2;


UPDATE [dbo].[Reports_ReportColumns] SET [INDEX] = 1 WHERE [INDEX] IS NULL

SELECT 1;

END TRY
BEGIN CATCH
    DECLARE @ErrorMessage VARCHAR(5000);
    DECLARE @ErrorSeverity INT;
    DECLARE @ErrorState INT;

    SELECT @ErrorMessage = 'Error Line: ' + CAST(ERROR_LINE() AS VARCHAR(50)) +' Error: ' + ERROR_MESSAGE(),
           @ErrorSeverity = ERROR_SEVERITY(),
           @ErrorState = ERROR_STATE();

    RAISERROR (@ErrorMessage, -- Message text.
               @ErrorSeverity, -- Severity.
               @ErrorState -- State.
               );
END CATCH;